/*************************************************************************************
In this file, I construct the exhaustive 238x238 matrices of all identifiable
bilateral portfolio assets starting from the CPIS and using a gravity-like model 
of bilateral portfolio holdings to derive the bilateral claims of non-CPIS countries
All details are in the Appendix: http://www.parisschoolofeconomics.eu/en/zucman-gabriel/ 
***************************************************************************************/


clear
set memory 1000m
set more off

cd "/Users/zucman/Documents/These/Missing Wealth/Working Paper July 2011/StataFiles/"


use "data_gravity.dta"


/**********************************************************************
***************************** Gravity-like Model **********************
**********************************************************************/


/*********************************************
*** Predicted shares from bilateral model ***
*********************************************/

drop if host==source

/* Fixed effects: host and year */
quietly tabulate year, generate(year_)
quietly tabulate host, generate(host_)

/* My definition of OFC for the regressions: the 42 countries reported in Table 2 of IMF (2000), except Switzerland */
gen ofc_source=0
replace ofc_source=1 if sifc_source==1
replace ofc_source=1 if sourcename=="Anguilla"|sourcename=="Liechtenstein"|sourcename=="Virgin Islands, British"|sourcename=="Monaco"|sourcename=="Cook Islands"|sourcename=="Niue"
replace ofc_source=1 if sourcename=="Ireland"|sourcename=="Luxembourg"|sourcename=="Hong Kong SAR of China"|sourcename=="Singapore"|sourcename=="Cyprus"
replace ofc_source=0 if sourcename=="British Indian Ocean Territory"|sourcename=="Dominica"|sourcename=="Grenada"|sourcename=="Liberia"|sourcename=="San Marino"|sourcename=="Uruguay"|sourcename=="Montserrat"|sourcename=="Palau"

save "temp.dta", replace
drop if ofc_source==0
keep source ofc_source
drop if source==source[_n-1]
rename ofc_source ofc_host
rename source host
sort host
save "temp_ofc.dta", replace

use "temp.dta"
sort host
merge host using "temp_ofc.dta", uniqusing
drop _merge
replace ofc_host=0 if ofc_host!=1

/* Benchmark Regression: Lane & Shambaugh (2010), appendix p. 2
i.e. same sample (CPIS excluding OFCs source and hosts)
and same controls (except for capital controls; I could not find the data) */
quietly eststo: reg logeqasset logdist gap_lon comlang_off col45 industrial loggap_gdp loggap_gdppc lat_source landlocked_source logpop_source loggdppc_source year_* host_* if ofc_source==0&ofc_host==0
quietly eststo: reg logdebtasset logdist gap_lon comlang_off col45 industrial loggap_gdp loggap_gdppc lat_source landlocked_source logpop_source loggdppc_source year_* host_* if ofc_source==0&ofc_host==0

/* Results: I get a somewhat smaller R2 than they do (0.76 instead of 0.79) and gap_long has the wrong sign.
So in what follows I drop it */

/* Regression augmented: Lane & Shambaugh, but full sample (ie including OFC) + interaction between OFC and host countries + drop gap_lon  */
xi I.ofc_source*I.host, prefix(_I)
set matsize 1000
quietly eststo: reg logeqasset logdist comlang_off col45 industrial loggap_gdp loggap_gdppc ofc_source lat_source landlocked_source logpop_source loggdppc_source  year_* host_* _IofcXhos_1_*
predict logeqp
quietly eststo: reg logdebtasset logdist comlang_off col45 industrial loggap_gdp loggap_gdppc ofc_source lat_source landlocked_source logpop_source loggdppc_source year_* host_* _IofcXhos_1_*
predict logdebtp

label variable logeqasset "Log equities"
label variable logdebtasset "Log debt"
label variable contig "Common border"
label variable comlang_off "Common language"
label variable col45 "Colony dummy"
label variable logdist "Log distance"
label variable gap_lon "Longitude gap"
label variable industrial "Industrial pair dummy"
label variable loggap_gdp "Log of GDP gap"
label variable loggap_gdppc "Log of GDP p.c. gap"
label variable lat_source "Latitude of source ctry"
label variable landlocked_source "Sce ctry landlocked"
label variable logpop_source "Log of sce ctry population"
label variable loggdppc_source "Log of sce ctry GDP p.c."
label variable ofc_source "Sce ctry OFC"

esttab, drop (_cons host_* year_* _IofcXhos_*) cells(b(star fmt(%9.3f)) se(par)) ar2 label legend

drop year_* host_* _Ihost_* _IofcXhos_* _Iofc_sourc_1

gen eqp=exp(logeqp)-1
gen debtp=exp(logdebtp)-1

/* some predicted values are very very slightly negative, I replace them by zero */
replace eqp=0 if eqp<0
replace debtp=0 if debtp<0


/* Comparison of predicted shares and true shares */

/* First: computation of allocated shares (i.e. excluding confidential (9996) and unallocated (9997) */
sort source year
by source year: egen toteqalloc=total(eqasset) if host!=9996&host!=9997
by source year: egen totdebtalloc=total(debtasset) if host!=9996&host!=9997
gen shareeqalloc=0
replace shareeqalloc=eqasset/toteqalloc
gen sharedebtalloc=0
replace sharedebtalloc=debtasset/totdebtalloc

/* Second: computation of predicted shares */
sort source year
by source year: egen toteqp=total(eqp)
by source year: egen totdebtp=total(debtp)
gen shareeqp=eqp/toteqp
gen sharedebtp=debtp/totdebtp

save "temp.dta", replace




/*************************************************************
*** Allocation of confidential + unallocated claims **********
*************************************************************/



replace eqasset=0 if eqasset==.
replace debtasset=0 if debtasset==.

save "temp.dta", replace


/****** First, I compute source-level aggregates, because these are bigger than total claims + unallocated claims 
--> So I want to put the residual in unallocated, and allocate the residual too **********/


/* a) I compute the raw sums */
sort source year
by source year: egen toteqasset=total(eqasset)
by source year: egen totdebtasset=total(debtasset)
save "temp.dta", replace

/* b) I compare the raw sums with aggregates from CPIS data */
use "data_totalequities.dta"
reshape long yr, i(ifscode) j(year)
rename yr sumeqasset
rename ifscode source
sort source year
save "temp_totequities.dta", replace

use "data_totaldebt.dta"
reshape long yr, i(ifscode) j(year)
rename yr sumdebtasset
rename ifscode source
sort source year
save "temp_totdebt.dta", replace

use "temp.dta"
merge source year using "temp_totequities.dta" "temp_totdebt.dta", uniqusing nosummary
drop _merge
drop cname
drop if year<2001
save "temp.dta", replace

/* Check that sums of aggregate is different than sums of all bilateral claims : 
table year, c(sum eqasset sum debtasset) format (%12.0fc)
sort source year host
table year if year!=year[_n-1], c(sum sumeqasset sum sumdebtasset) format (%12.0fc) */


/* c) CONCLUSION = In the IMF CPIS raw datafiles (August 2010), the sum for each source country of its foreign claims is not always
equal to the reported total, i.e. some unllocated claims are missing from the "unallocated line".
I add them here >> this augments the line unallocated (9997) */

gen missingeq=0
replace missingeq=sumeqasset-toteqasset
gen missingdebt=0
replace missingdebt=sumdebtasset-totdebtasset
replace eqasset=eqasset+missingeq if host==9997&missingeq!=.
replace debtasset=debtasset+missingdebt if host==9997&missingdebt!=.

drop sumeqasset sumdebtass missingeq missingdebt
save "temp.dta", replace


/****** Second: allocation of unallocated+confidential *********/ 


/* a) I assume that all unallocated + confidential claims are vis-a-vis countries for which A_{ij} is either missing or zero
or Andorra, or Liechtenstein or Monaco (in most cases A_{ij} is zero for these countries but sometimes not, which introduces 
spurrious variation in the allocation of the unallocated claims.
I also disregard San Marino, a clear outlier.
(NB: I have replaced all missing values by zeros at the beginning of this section) */ 

gen confidentialeq=0
replace confidentialeq=1 if (eqasset==0|hostname=="Andorra"|hostname=="Liechtenstein"|hostname=="Monaco")&toteqasset!=.&toteqasset!=0&hostname!="San Marino"

gen confidentialdebt=0
replace confidentialdebt=1 if (debtasset==0|hostname=="Andorra"|hostname=="Liechtenstein"|hostname=="Monaco")&totdebtasset!=.&totdebtasset!=0&hostname!="San Marino"

/* b): Prorate source-level confidential and unallocated totals, 
using the predicted shares from the bilateral regression for missing A_{ij} pairs */

/* i) Generate predicted share of each missing country (rescaled such that sum of predicted shares = 1) */
gen eqpconf=eqp*confidentialeq
gen debtpconf=debtp*confidentialdebt

sort source year
by source year: egen toteqpconf=total(eqpconf)
by source year: egen totdebtpconf=total(debtpconf)

gen shareeqpconf=eqpconf/toteqpconf
replace shareeqpconf=0 if shareeqpconf==.
gen sharedebtpconf=debtpconf/totdebtpconf
replace sharedebtpconf=0 if sharedebtpconf==.

save "temp.dta", replace

/* b) generate colums with amount of confidential (9996) + unallocated (9997) equity + debt claims */

keep if host==9996|host==9997
keep source year host eqasset debtasset
sort source year
by source year: egen totunalloceq=total(eqasset)
by source year: egen totunallocdebt=total(debtasset)
drop host eqasset debtasset
sort year source
drop if source==source[_n-1]
save "temp_unalloc.dta", replace

/* c) Merge these column with my dataset */

use "temp.dta"
sort year source
merge year source using "temp_unalloc.dta", uniqusing
drop _merge

/* d) Now create predicted claims for all countries non SEFER */

gen unalloceq=0
replace unalloceq=shareeqpconf*totunalloceq if source!=9999
gen unallocdebt=0
replace unallocdebt=sharedebtpconf*totunallocdebt if source!=9999

sort source year
save "temp.dta", replace

/* e) I create an augmented asset line, equal to the raw asset line + allocated confidential+unalloc
I set the unallocated and confidential lines to zero */

gen augmeqasset=eqasset
replace augmeqasset=eqasset+unalloceq if host!=9996&host!=9997&source!=9999
replace augmeqasset=0 if host==9996|host==9997

gen augmdebtasset=debtasset
replace augmdebtasset=debtasset+unallocdebt if host!=9996&host!=9997&source!=9999
replace augmdebtasset=0 if host==9996|host==9997

/* f) For SEFER, I simply prorate unallocad+confidential claims following the allocated claims pattern */

replace augmeqasset=augmeqasset+shareeqalloc*totunalloceq if source==9999
replace augmdebtasset=augmdebtasset+sharedebtalloc*totunallocdebt if source==9999


save "temp.dta", replace


/* Check=
sort source year host
table year, c(sum eqasset sum augmeqasset sum debtasset sum augmdebtasset) format(%12.0fc) 
*/

drop sharedebtpconf shareeqpconf totdebtpconf toteqpconf debtpconf eqpconf confidentialdebt confidentialeq
save "temp.dta", replace



/*****************************************************************************
******** Allocation of Cayman Islands non-bank sector ************************
*******************************************************************************/


/* Method: I compute the U.S. share from the gravity model like model (see supra) */

/* I use the predicted U.S. shares + TIC data on U.S. assets held by the Cayman Islands to infer the complete Caymanian holdings */
replace augmeqasset=57128 if source==377&host==111&year==2001
replace augmeqasset=65171 if source==377&host==111&year==2002
replace augmeqasset=116663 if source==377&host==111&year==2003
replace augmeqasset=139966 if source==377&host==111&year==2004
replace augmeqasset=165426 if source==377&host==111&year==2005
replace augmeqasset=220741 if source==377&host==111&year==2006
replace augmeqasset=329207 if source==377&host==111&year==2007
replace augmeqasset=212798 if source==377&host==111&year==2008

replace augmdebtasset=102419 if source==377&host==111&year==2001
replace augmdebtasset=128419 if source==377&host==111&year==2002
replace augmdebtasset=159794 if source==377&host==111&year==2003
replace augmdebtasset=260587 if source==377&host==111&year==2004
replace augmdebtasset=279807 if source==377&host==111&year==2005
replace augmdebtasset=425252 if source==377&host==111&year==2006
replace augmdebtasset=522261 if source==377&host==111&year==2007
replace augmdebtasset=493578 if source==377&host==111&year==2008

sort source year
save "temp.dta", replace

forvalues t=2001(1)2008 {
gen caymantoteq`t'=augmeqasset/shareeqp if year==`t'&source==377&host==111
gen caymantotdebt`t'=augmdebtasset/sharedebtp if year==`t'&source==377&host==111
}

keep if host==111&source==377
keep source caymantoteq* caymantotdebt*
gen id=[_n]
reshape long caymantoteq caymantotdebt, i(id) j(year)
drop if caymantoteq==.|caymantotdebt==.
drop id
sort source year
save "temp_caymantot.dta", replace

use "temp.dta"
merge source year using "temp_caymantot.dta", uniqusing
drop _merge

replace augmeqasset=caymantoteq*shareeqp if source==377
replace augmdebtasset=caymantotdebt*sharedebtp if source==377


/* CHECK 
table year if source==377, c(sum augmeqasset mean caymantoteq sum augmdebtasset mean caymantotdebt)
 */
drop caymantotdebt caymantoteq

save "temp.dta", replace



/*****************************************************************************
****************** Allocation of other CPIS countries ************************
*****************************************************************************/


/* Filling the gap for CPIS countries that have not always reported their data */
replace toteqasset=2443 if sourcename=="Bahrain"&year==2002
replace toteqasset=3539 if sourcename=="Bahrain"&year==2003
replace toteqasset=463 if sourcename=="Barbados"&year==2001
replace toteqasset=427 if sourcename=="Barbados"&year==2002
replace toteqasset=23 if sourcename=="Gibraltar"&year==2001
replace toteqasset=21 if sourcename=="Gibraltar"&year==2002
replace toteqasset=31 if sourcename=="Gibraltar"&year==2003
replace toteqasset=17 if sourcename=="India"&year==2001
replace toteqasset=16 if sourcename=="India"&year==2002
replace toteqasset=23 if sourcename=="India"&year==2003
replace toteqasset=3185 if sourcename=="Kuwait"&year==2001
replace toteqasset=2937 if sourcename=="Kuwait"&year==2002
replace toteqasset=109 if sourcename=="Latvia"&year==2001
replace toteqasset=101 if sourcename=="Latvia"&year==2002
replace toteqasset=146 if sourcename=="Latvia"&year==2003
replace toteqasset=183 if sourcename=="Latvia"&year==2004
replace toteqasset=223 if sourcename=="Latvia"&year==2005
replace toteqasset=302 if sourcename=="Mexico"&year==2001
replace toteqasset=278 if sourcename=="Mexico"&year==2002

replace augmeqasset=toteqasset*shareeqp if sourcename=="Bahrain"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="Bahrain"&year==2003
replace augmeqasset=toteqasset*shareeqp if sourcename=="Barbados"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="Barbados"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="Gibraltar"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="Gibraltar"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="Gibraltar"&year==2003
replace augmeqasset=toteqasset*shareeqp if sourcename=="India"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="India"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="India"&year==2003
replace augmeqasset=toteqasset*shareeqp if sourcename=="Kuwait"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="Kuwait"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="Latvia"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="Latvia"&year==2002
replace augmeqasset=toteqasset*shareeqp if sourcename=="Latvia"&year==2003
replace augmeqasset=toteqasset*shareeqp if sourcename=="Latvia"&year==2004
replace augmeqasset=toteqasset*shareeqp if sourcename=="Latvia"&year==2005
replace augmeqasset=toteqasset*shareeqp if sourcename=="Mexico"&year==2001
replace augmeqasset=toteqasset*shareeqp if sourcename=="Mexico"&year==2002

replace totdebtasset=11822 if sourcename=="Bahrain"&year==2002
replace totdebtasset=15706 if sourcename=="Bahrain"&year==2003
replace totdebtasset=789 if sourcename=="Barbados"&year==2001
replace totdebtasset=701 if sourcename=="Barbados"&year==2002
replace totdebtasset=281 if sourcename=="Gibraltar"&year==2001
replace totdebtasset=316 if sourcename=="Gibraltar"&year==2002
replace totdebtasset=425 if sourcename=="Gibraltar"&year==2003
replace totdebtasset=19 if sourcename=="India"&year==2001
replace totdebtasset=24 if sourcename=="India"&year==2002
replace totdebtasset=31 if sourcename=="India"&year==2003
replace totdebtasset=3216 if sourcename=="Kuwait"&year==2001
replace totdebtasset=4171 if sourcename=="Kuwait"&year==2002
replace totdebtasset=678 if sourcename=="Latvia"&year==2001
replace totdebtasset=773 if sourcename=="Latvia"&year==2002
replace totdebtasset=1033 if sourcename=="Latvia"&year==2003
replace totdebtasset=1258 if sourcename=="Latvia"&year==2004
replace totdebtasset=1381 if sourcename=="Latvia"&year==2005
replace totdebtasset=3118 if sourcename=="Mexico"&year==2001
replace totdebtasset=3519 if sourcename=="Mexico"&year==2002

replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Bahrain"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Bahrain"&year==2003
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Barbados"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Barbados"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Gibraltar"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Gibraltar"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Gibraltar"&year==2003
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="India"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="India"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="India"&year==2003
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Kuwait"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Kuwait"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Latvia"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Latvia"&year==2002
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Latvia"&year==2003
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Latvia"&year==2004
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Latvia"&year==2005
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Mexico"&year==2001
replace augmdebtasset=totdebtasset*sharedebtp if sourcename=="Mexico"&year==2002


/* The missing Netherlands SFIs */
replace toteqasset=toteqasset+3286 if sourcename=="Netherlands"&year==2001 
replace toteqasset=toteqasset+2601 if sourcename=="Netherlands"&year==2002
replace toteqasset=toteqasset+5161 if sourcename=="Netherlands"&year==2003 
replace toteqasset=toteqasset+3665 if sourcename=="Netherlands"&year==2004 
replace toteqasset=toteqasset+4707 if sourcename=="Netherlands"&year==2005 
replace toteqasset=toteqasset+9594 if sourcename=="Netherlands"&year==2006 
replace toteqasset=toteqasset+24281 if sourcename=="Netherlands"&year==2007 
replace toteqasset=toteqasset+21706 if sourcename=="Netherlands"&year==2008 

replace totdebtasset=totdebtasset+20984 if sourcename=="Netherlands"&year==2001 
replace totdebtasset=totdebtasset+16611 if sourcename=="Netherlands"&year==2002
replace totdebtasset=totdebtasset+32958 if sourcename=="Netherlands"&year==2003 
replace totdebtasset=totdebtasset+41826 if sourcename=="Netherlands"&year==2004 
replace totdebtasset=totdebtasset+58612 if sourcename=="Netherlands"&year==2005 
replace totdebtasset=totdebtasset+48136 if sourcename=="Netherlands"&year==2006 
replace totdebtasset=totdebtasset+66732 if sourcename=="Netherlands"&year==2007 
replace totdebtasset=totdebtasset+62497 if sourcename=="Netherlands"&year==2008 


replace augmeqasset=toteqasset*shareeqalloc if sourcename=="Netherlands"
replace augmdebtasset=totdebtasset*sharedebtalloc if sourcename=="Netherlands"

save "temp.dta", replace



/*****************************************************************************
********************************* Allocation of CHINA ************************
*****************************************************************************/


/* Total Chinese assets given by private portfolios + 85% of reserves
U.S. assets owned by China given by TIC survey hence U.S. share
I assume that Non-US shares given by share of each individual country in total SEFER+SSIO investments in non-US countries */


/* Total and U.S. assets */

replace toteqasset=5473 if sourcename=="China, P.R."&year==2001
replace toteqasset=3635 if sourcename=="China, P.R."&year==2002
replace toteqasset=3329 if sourcename=="China, P.R."&year==2003
replace toteqasset=3963 if sourcename=="China, P.R."&year==2004
replace toteqasset=4374 if sourcename=="China, P.R."&year==2005
replace toteqasset=20392 if sourcename=="China, P.R."&year==2006
replace toteqasset=104236 if sourcename=="China, P.R."&year==2007
replace toteqasset=94676 if sourcename=="China, P.R."&year==2008

replace totdebtasset=208304 if sourcename=="China, P.R."&year==2001
replace totdebtasset=286588 if sourcename=="China, P.R."&year==2002
replace totdebtasset=403917 if sourcename=="China, P.R."&year==2003
replace totdebtasset=607307 if sourcename=="China, P.R."&year==2004
replace totdebtasset=809352 if sourcename=="China, P.R."&year==2005
replace totdebtasset=1153808 if sourcename=="China, P.R."&year==2006
replace totdebtasset=1480617 if sourcename=="China, P.R."&year==2007
replace totdebtasset=1811330 if sourcename=="China, P.R."&year==2008

gen useqassetchina=3906 if sourcename=="China, P.R."&year==2001
replace useqassetchina=2685 if sourcename=="China, P.R."&year==2002
replace useqassetchina=2419 if sourcename=="China, P.R."&year==2003
replace useqassetchina=2763 if sourcename=="China, P.R."&year==2004
replace useqassetchina=3268 if sourcename=="China, P.R."&year==2005
replace useqassetchina=14117 if sourcename=="China, P.R."&year==2006
replace useqassetchina=72075 if sourcename=="China, P.R."&year==2007
replace useqassetchina=71720 if sourcename=="China, P.R."&year==2008

gen usdebtassetchina=148651 if sourcename=="China, P.R."&year==2001
replace usdebtassetchina=211661 if sourcename=="China, P.R."&year==2002
replace usdebtassetchina=293478 if sourcename=="China, P.R."&year==2003
replace usdebtassetchina=423443 if sourcename=="China, P.R."&year==2004
replace usdebtassetchina=604667 if sourcename=="China, P.R."&year==2005
replace usdebtassetchina=798747 if sourcename=="China, P.R."&year==2006
replace usdebtassetchina=1023785 if sourcename=="China, P.R."&year==2007
replace usdebtassetchina=1372144 if sourcename=="China, P.R."&year==2008


/* Total non U.S. assets of China */

gen nonuseqassetchina=toteqasset-useqassetchina if sourcename=="China, P.R."
gen nonusdebtassetchina=totdebtasset-usdebtassetchina if sourcename=="China, P.R."

save "temp.dta", replace

/* Share of SEFER+SSIO in each non-US non-China country  */

drop if source!=9999
drop if host==111|hostname=="China, P.R."
keep source year host augmeqasset augmdebtasset
sort year
by year: egen toteqnonus=total(augmeqasset)
by year: egen totdebtnonus=total(augmdebtasset)
gen shareeqsefernonus=augmeqasset/toteqnonus
gen sharedebtsefernonus=augmdebtasset/totdebtnonus
keep year host shareeqsefernonus sharedebtsefernonus
sort host year
save "temp_sefernonus.dta", replace

use "temp.dta"
sort host year
merge host year using "temp_sefernonus.dta", uniqusing
drop _merge

/* Computation of China's bilateral assets */

replace augmeqasset=useqassetchina if sourcename=="China, P.R."&host==111
replace augmdebtasset=usdebtassetchina if sourcename=="China, P.R."&host==111
replace shareeqalloc=augmeqasset/toteqasset if sourcename=="China, P.R."&host==111
replace sharedebtalloc=augmdebtasset/totdebtasset if sourcename=="China, P.R."&host==111

replace augmeqasset=shareeqsefernonus*nonuseqassetchina if sourcename=="China, P.R."&host!=111
replace augmdebtasset=sharedebtsefernonus*nonusdebtassetchina if sourcename=="China, P.R."&host!=111


/* CHECK 
table year if sourcename=="China, P.R.", c(sum augmeqasset sum augmdebtasset)
*/

drop nonusdebtassetchina nonuseqassetchina usdebtassetchina useqassetchina shareeqsefernonus sharedebtsefernonus

save "temp.dta", replace



/*****************************************************************************
********************************* Allocation of ME OIL ************************
*****************************************************************************/


/* Total U.S. (onshore) assets given by TIC survey
Following literature assumptions on U.S. share (70% in 2002 and declining 2 pct points per year >> 56% in 2008 
For non-US shares: bilateral model (because probably invested differently than reserve assets) 
Further, I assume for convenience that ME oil exporters can be representend by Saudi Arabia ifscode 456 (since we don't have
bilateral holdings anyway in the TIC survey)
So Saudi Arabia = total assets of non-CPIS middle east oil exporters */


/* Total and U.S. assets */

replace toteqasset=58202 if sourcename=="Saudi Arabia"&year==2001
replace toteqasset=50307 if sourcename=="Saudi Arabia"&year==2002
replace toteqasset=79471 if sourcename=="Saudi Arabia"&year==2003
replace toteqasset=112398 if sourcename=="Saudi Arabia"&year==2004
replace toteqasset=137271 if sourcename=="Saudi Arabia"&year==2005
replace toteqasset=188614 if sourcename=="Saudi Arabia"&year==2006
replace toteqasset=225333 if sourcename=="Saudi Arabia"&year==2007
replace toteqasset=143485 if sourcename=="Saudi Arabia"&year==2008

replace totdebtasset=66574 if sourcename=="Saudi Arabia"&year==2001
replace totdebtasset=65239 if sourcename=="Saudi Arabia"&year==2002
replace totdebtasset=58344 if sourcename=="Saudi Arabia"&year==2003
replace totdebtasset=100009 if sourcename=="Saudi Arabia"&year==2004
replace totdebtasset=126877 if sourcename=="Saudi Arabia"&year==2005
replace totdebtasset=180083 if sourcename=="Saudi Arabia"&year==2006
replace totdebtasset=285918 if sourcename=="Saudi Arabia"&year==2007
replace totdebtasset=369851 if sourcename=="Saudi Arabia"&year==2008

gen usshareoil=0.7 if sourcename=="Saudi Arabia"&year==2001
replace usshareoil=0.68 if sourcename=="Saudi Arabia"&year==2002
replace usshareoil=0.66 if sourcename=="Saudi Arabia"&year==2003
replace usshareoil=0.64 if sourcename=="Saudi Arabia"&year==2004
replace usshareoil=0.62 if sourcename=="Saudi Arabia"&year==2005
replace usshareoil=0.60 if sourcename=="Saudi Arabia"&year==2006
replace usshareoil=0.58 if sourcename=="Saudi Arabia"&year==2007
replace usshareoil=0.56 if sourcename=="Saudi Arabia"&year==2008


gen usoileqasset=usshareoil*toteqasset if sourcename=="Saudi Arabia"
gen usoildebtasset=usshareoil*totdebtasset if sourcename=="Saudi Arabia"
gen nonusoileqasset=toteqasset-usoileqasset if sourcename=="Saudi Arabia"
gen nonusoildebtasset=totdebtasset-usoildebtasset if sourcename=="Saudi Arabia"

save "temp.dta", replace

/* non-US shares */
/* I rescale the predicted shares */
keep if host==111
keep source year shareeqp sharedebtp
rename shareeqp shareequsp 
rename sharedebtp sharedebtusp
sort source year
save "temp_shareusp.dta", replace
use "temp.dta"
sort source year
merge source year using "temp_shareusp.dta", uniqusing
drop _merge
gen shareeqpnonus=shareeqp/(1-shareequsp)
gen sharedebtpnonus=sharedebtp/(1-sharedebtusp)

replace augmeqasset=shareeqpnonus*nonusoileqasset if sourcename=="Saudi Arabia"&host!=111
replace augmdebtasset=sharedebtpnonus*nonusoildebtasset if sourcename=="Saudi Arabia"&host!=111
replace augmeqasset=usshareoil*toteqasset if sourcename=="Saudi Arabia"&host==111
replace augmdebtasset=usshareoil*totdebtasset if sourcename=="Saudi Arabia"&host==111

drop usshareoil usoileqasset usoildebtasset nonusoileqasset nonusoildebtasset
save "temp.dta", replace

/* CHECK 
table year if sourcename=="Saudi Arabia", c(sum augmeqasset sum augmdebtasset) 
*/



/*****************************************************************************
***************************** Allocation of other non CPIS *******************
*****************************************************************************/


/* I -- Private assets */

/* Total portfolio assets of non-CPIS countries are given
a) by Lane and Milesi-Ferretti (2007) EWNII database updated August 2009
b) When aportif_debt is missing, I take 20% of adebt (20% = unweighted mean value of aportif_debt/adebt 
for the sample of countries that have both data in the EWNII August 2009. NB: slight upward trend over 2001-2007)
c) for countries not in the database: augmented CPIS-derived liabilities
d) Bilateral allocation = gravity model  */

/*
Data from LMF
*/


sort source year
save "temp.dta", replace

use "data_lanemilesi09.dta"
drop if year<2001
rename ifscode source
sort source year
save "temp_ewnii.dta", replace

use "temp.dta"
merge source year using "temp_ewnii.dta", uniqusing keep(aequity adebt aportif_debt ewnii)
/* ewnii = dummy for country in EWNII */
rename ewnii ewnii_source
sort source host year
replace ewnii_source=1 if ewnii==.&ewnii_source[_n-1]==1&year==2008
drop _merge


/* When aportif_debt missing, use 0.2 x adebt */
replace aportif_debt=0.2*adebt if aportif_debt==.&adebt!=. 

/* For 2008, I assume debt level = 0.913* 2007 level, and equity level = 0.575 times 2007 level (this is what we have on average for CPIS private portfolios) */
sort source host year
replace aequity=0.575*aequity[_n-1] if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433&year==2008
replace aportif_debt=0.913*aportif_debt[_n-1] if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433&year==2008

replace toteqasset=aequity if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433
replace totdebtasset=aportif_debt  if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433

save "temp.dta", replace

/* 
Derived liabilities from (augmented) CPIS 
*/

sort host year
by host year: egen augmeqliab=total(augmeqasset)
by host year: egen augmdebtliab=total(augmdebtasset)
keep host year augmeqliab augmdebtliab
drop if augmeqliab[_n]==augmeqliab[_n-1]
rename host source
sort source year
save "temp_derived.dta", replace


use "temp.dta"
sort source year
merge source year using "temp_derived.dta", uniqusing
drop _merge

replace toteqasset=augmeqliab if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=429&source!=433&source!=466&source!=9998&aequity==.
/* NB: important to subtract IOs (9998) which have derived liab, but whose assets are already counted in SEFER 9999 */
replace totdebtasset=augmdebtliab if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433&source!=9998&aportif_debt==.

save "temp.dta", replace


/* 
Bilateral estimates: for private holdings, gravity model
*/

replace augmeqasset=shareeqp*toteqasset if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433
replace augmdebtasset=sharedebtp*totdebtasset if cpis!=1&sourcename!="China, P.R."&source!=449&source!=453&source!=456&source!=466&source!=429&source!=433
save "temp.dta", replace


/* II - Reserve of non-CPIS countries: I use IFS data and assume 75% invested in securities
(74% in debt and 1% in equity) */


use "data_foreignexchange.dta"
reshape long yr, i(ifscode) j(year)
rename yr reserveIFS
rename ifscode source
drop cname
drop if year<2001
drop if source==163|source==309|source==758|source==759|source==967 /* Regional organizations */
sort source year
save "temp_missingreserve", replace

use "temp.dta"
sort source year
merge source year using "temp_missingreserve", uniqusing
drop _merge
/* Taiwan not in IFS... So I use figure from Taiwanese IIP
(this is not fully comparable to IFS line 1d.d but close. */
replace reserveIFS=126572 if source==528&year==2001
replace reserveIFS=166046 if source==528&year==2002
replace reserveIFS=211139 if source==528&year==2003
replace reserveIFS=246561 if source==528&year==2004
replace reserveIFS=257952 if source==528&year==2005
replace reserveIFS=270840 if source==528&year==2006
replace reserveIFS=275027 if source==528&year==2007
replace reserveIFS=296389 if source==528&year==2008


/* I create the total missing reserves */

gen debtreserveIFS=0.74*reserveIFS
gen eqreserveIFS=0.01*reserveIFS

save "temp.dta", replace

sort source year
drop if year==year[_n-1]
/* I assume that all CPIS countries participate in SEFER, so missing reserves = non CPIS non oil non China */
drop if cpis==1|source==924|source==449|source==453|source==456|source==466|source==433|source==429|source==429|source==433
sort year
by year: egen missingeqres=total(eqreserveIFS)
by year: egen missingdebtres=total(debtreserveIFS)
keep year missingeqres missingdebtres
sort year
drop if year==year[_n-1]
save "temp_missingreserve2", replace

use "temp.dta"
sort year
merge year using "temp_missingreserve2", uniqusing
drop _merge
save "temp.dta", replace

/* I assume that they all follow the same SEFER-SSIO pattern */

drop if source!=9999

keep year host augmeqasset augmdebtasset
sort year
by year: egen toteq=total(augmeqasset)
by year: egen totdebt=total(augmdebtasset)
gen shareeqsefer=augmeqasset/toteq
gen sharedebtsefer=augmdebtasset/totdebt
keep year host shareeqsefer sharedebtsefer
sort host year
save "temp_sefer.dta", replace

use "temp.dta"
sort host year
merge host year using "temp_sefer.dta", uniqusing
drop _merge

save "temp.dta", replace
 
/* I create a new "source" country 9994 corresponding to the missing SEFER (non-oil and non China) */
gen othereqreserve=shareeqsefer*missingeqres if year==2001
replace othereqreserve=shareeqsefer*missingeqres if year==2002
replace othereqreserve=shareeqsefer*missingeqres if year==2003
replace othereqreserve=shareeqsefer*missingeqres if year==2004
replace othereqreserve=shareeqsefer*missingeqres if year==2005
replace othereqreserve=shareeqsefer*missingeqres if year==2006
replace othereqreserve=shareeqsefer*missingeqres if year==2007
replace othereqreserve=shareeqsefer*missingeqres if year==2008

gen otherdebtreserve=sharedebtsefer*missingdebtres if year==2001
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2002
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2003
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2004
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2005
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2006
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2007
replace otherdebtreserve=sharedebtsefer*missingdebtres if year==2008

save "temp.dta", replace


keep host year othereqreserve otherdebtreserve
sort host year
drop if year==year[_n-1]
drop if host==.
gen source=9994
rename othereqreserve augmeqasset
rename otherdebtreserve augmdebtasset
sort source host year
save "temp_otherreserve.dta", replace

use "temp.dta"
sort source host year
append using "temp_otherreserve.dta"

save "temp.dta", replace




/**************************************************
********** EWNII and derived liabilities **********
**************************************************/


/* Three methods for liabilities: 1) for EWNII countries, use data from EWNII 
[1bis) Correction to EWNII data: Netherlands, CPIS<liab, IOs]
2) for non EWNII countries, use CPIS-augmented derived liab
3) for non EWNII for which I have independent information (BIS international debt, fund NAV...)
use max(derived liab, independent info) */



/* A- Bringing back liabilities for host countries */

/* 1) EWNII */

use "data_lanemilesi09.dta"
rename ifscode host
sort host year
save "temp_ewnii.dta", replace

use "temp.dta"
sort host year
merge host year using "temp_ewnii.dta", uniqusing keep(lequity lportif_debt ewnii)
rename ewnii ewnii_host
sort host source year
replace ewnii_host=1 if ewnii_host==.&ewnii_host[_n-1]==1&year==2008
drop _merge

save "temp.dta", replace



/* For 2008, I use IMF IIPs */

use "data_portfolioequities.dta"
reshape long yr, i(ifscode) j(year)
rename yr lequity_IMF
rename ifscode host
drop if host==163
sort host year
save "temp_eqliab2008.dta", replace
use "temp.dta"
sort host year
merge host year using "temp_eqliab2008.dta", uniqusing
drop _merge
save "temp.dta", replace

use "data_portfoliodebt.dta"
reshape long yr, i(ifscode) j(year)
rename yr lportif_debt_IMF
rename ifscode host
drop if host==163
sort host year
save "temp_debtliab2008.dta", replace
use "temp.dta"
sort host year
merge host year using "temp_debtliab2008.dta", uniqusing 
drop _merge
save "temp.dta", replace


/* Correction for 2008 */
replace lportif_debt=lportif_debt_IMF if year==2008&lportif_debt_IMF!=.
replace lequity=lequity_IMF if year==2008&lequity_IMF!=.


/* When IIP is unavailable, I assume that lequity=0.58*lequity2007 and lportif_debt=0.97*lportif_debt2007 
(this is equal to the evolution of total assets, table A1 col. 13) */


sort host source year
replace lequity=0.57*lequity[_n-1] if year==2008&lequity==.
replace lportif_debt=0.95*lportif_debt[_n-1] if year==2008&lportif_debt==.

rename lequity lequity_host
rename lportif_debt lportif_debt_host
drop if year<2001
drop if year==2009
drop if host==163 /* Euro area */


save "temp.dta", replace



/* 1bis) Correction for EWNII data */

/* First of all: correction for Netherlands SFIs */

gen eqliab_host=lequity_host
gen debtliab_host=lportif_debt_host

replace eqliab_host=lequity_host+6635 if hostname=="Netherlands"&year==2001
replace eqliab_host=lequity_host+8177 if hostname=="Netherlands"&year==2002
replace eqliab_host=lequity_host+10898 if hostname=="Netherlands"&year==2003
replace eqliab_host=lequity_host+11989 if hostname=="Netherlands"&year==2004
replace eqliab_host=lequity_host+11138 if hostname=="Netherlands"&year==2005
replace eqliab_host=lequity_host+42538 if hostname=="Netherlands"&year==2006
replace eqliab_host=lequity_host+18459 if hostname=="Netherlands"&year==2007
replace eqliab_host=lequity_host+14309 if hostname=="Netherlands"&year==2008

replace debtliab_host=lportif_debt_host+258296 if hostname=="Netherlands"&year==2001
replace debtliab_host=lportif_debt_host+318308 if hostname=="Netherlands"&year==2002
replace debtliab_host=lportif_debt_host+424262 if hostname=="Netherlands"&year==2003
replace debtliab_host=lportif_debt_host+446463 if hostname=="Netherlands"&year==2004
replace debtliab_host=lportif_debt_host+382669 if hostname=="Netherlands"&year==2005
replace debtliab_host=lportif_debt_host+473133 if hostname=="Netherlands"&year==2006
replace debtliab_host=lportif_debt_host+607871 if hostname=="Netherlands"&year==2007
replace debtliab_host=lportif_debt_host+592400 if hostname=="Netherlands"&year==2008


/* Second: use IMF Data when no portfolio debt in EWNII */

replace debtliab_host=lportif_debt_IMF if lportif_debt_host==.&lportif_debt_IMF!=.


/* Third: correction for EWNII reported liab < raw CPIS derived liab */

sort host year
by host year: egen rawderivedeq=total(eqasset)
by host year: egen rawderiveddebt=total(debtasset)

gen missingliabeq=0
replace missingliabeq=rawderivedeq-eqliab_host if rawderivedeq>eqliab_host
gen missingliabdebt=0
replace missingliabdebt=rawderiveddebt-debtliab_host if rawderiveddebt>debtliab_host

replace eqliab_host=eqliab_host+missingliabeq
replace debtliab_host=debtliab_host+missingliabdebt

/* Fourth : Add liabilities for international organizations
Debt = BIS data
Equity = raw CPIS derived */

replace eqliab_host=rawderivedeq if host==9998
replace debtliab_host=382674 if host==9998&year==2001
replace debtliab_host=436469 if host==9998&year==2002
replace debtliab_host=505950 if host==9998&year==2003
replace debtliab_host=554524 if host==9998&year==2004
replace debtliab_host=543238 if host==9998&year==2005
replace debtliab_host=586554 if host==9998&year==2006
replace debtliab_host=659142 if host==9998&year==2007
replace debtliab_host=657321 if host==9998&year==2008


/* 2) Augmented derived liabilities for non EWNII nations */

sort host year
by host year: egen derivedeqliab_host=total(augmeqasset) 
by host year: egen deriveddebtliab_host=total(augmdebtasset)

replace eqliab_host=derivedeqliab_host if eqliab_host==.&host!=9996&host!=9997&host!=9998&host!=9999
replace debtliab_host=deriveddebtliab_host if debtliab_host==.&host!=9996&host!=9997&host!=9998&host!=9999


save "temp.dta", replace


/* 3) Adding my data for SIFC (fund NAV, common stocks U.S., BIS international debt) */


use "data_sifcliab.dta"
reshape long bisdebt fundnav uscommstk, i(source) j(year)
rename source host
sort host year
save "temp_sifcliabforstata_LONG.dta", replace

use "temp.dta"
merge host year using "temp_sifcliabforstata_LONG.dta", uniqusing
drop _merge

gen repeq=fundnav+uscommstk
egen sifceqliab_host=rowmax(derivedeqliab_host repeq)
drop repeq
egen sifcdebtliab_host=rowmax(deriveddebtliab_host bisdebt)
replace eqliab_host=sifceqliab_host if fundnav!=.
replace debtliab_host=sifcdebtliab_host if bisdebt!=.

save "temp.dta", replace



/* B - Liability data for source countries */


keep host year lequity_host lportif_debt_host derivedeqliab_host deriveddebtliab_host eqliab_host debtliab_host
rename host source
rename lequity_host lequity_source
rename lportif_debt_host lportif_debt_source
rename derivedeqliab_host derivedeqliab_source
rename deriveddebtliab_host deriveddebtliab_source
rename eqliab_host eqliab_source
rename debtliab_host debtliab_source
sort source year
collapse (first) lequity_source lportif_debt_source derivedeqliab_source deriveddebtliab_source eqliab_source debtliab_source, by(year source)
sort source year
save "temp_liab.dta", replace

use "temp.dta"
sort source year
merge source year using "temp_liab.dta", uniqusing
drop _merge


/* Generate gap reported liab - derived liab */

gen gapeq_host=eqliab_host-derivedeqliab_host
gen gapdebt_host=debtliab_host-deriveddebtliab_host

gen gapeq_source=eqliab_source-derivedeqliab_source
gen gapdebt_source=debtliab_source-deriveddebtliab_source

save "data_full_matrices.dta", replace



/* Check that the gap computed on source countries is the same as computed on host countries 
sort host year  
collapse (first) hostname eqliab_host derivedeqliab_host gapeq_host debtliab_host deriveddebtliab_host gapdebt_host, by(host year)
table year, c(sum gapeq_host sum gapdebt_host) format(%12.0g)
save "temp_gap_host.dta", replace
use "data_full_matrices.dta"
sort source year
collapse (first) sourcename eqliab_source derivedeqliab_source gapeq_source debtliab_source deriveddebtliab_source gapdebt_source, by(source year)
table year, c(sum gapeq_source sum gapdebt_source) format(%12.0g)
save "temp_gap_source.dta", replace
*/



